@@ -13,10 +13,8 @@ use qp2p::IncomingMessages;
 use sn_data_types::PublicKey;
 use sn_messaging::{
     client::{ClientMsg, Event, ProcessMsg},
-    section_info::{
-        Error as SectionInfoError, GetSectionResponse, Message as SectionInfoMsg, SectionInfo,
-    },
-    MessageId, MessageType, WireMsg,
+    section_info::{Error as SectionInfoError, GetSectionResponse, SectionInfoMsg},
+    MessageId, MessageType, SectionAuthorityProvider, WireMsg,
 };
 use std::{
     collections::{BTreeMap, BTreeSet},
@@ -27,8 +25,8 @@ impl Session {
     /// Remove a pending transfer sender from the listener map
     pub async fn remove_pending_transfer_sender(&self, msg_id: &MessageId) -> Result<(), Error> {
         let pending_transfers = self.pending_transfers.clone();
-        debug!("Pending transfers at this point: {:?}", pending_transfers);
-        let mut listeners = pending_transfers.lock().await;
+        let mut listeners = pending_transfers.write().await;
+        debug!("Pending transfers at this point: {:?}", listeners);
         let _ = listeners
             .remove(msg_id)
             .ok_or(Error::NoTransferValidationListener)?;
@@ -124,20 +122,20 @@ impl Session {
                 Ok(())
             }
             SectionInfoMsg::GetSectionResponse(GetSectionResponse::SectionInfoUpdate(
-                SectionInfoError::TargetSectionInfoOutdated(info),
+                SectionInfoError::TargetSectionInfoOutdated(sap),
             )) => {
-                debug!("Updated section info received: {:?}", info);
-                self.update_session_info(info).await?;
+                debug!("Updated section info received: {:?}", sap);
+                self.update_session_info(sap).await?;
                 Ok(())
             }
-            SectionInfoMsg::GetSectionResponse(GetSectionResponse::Redirect(elders)) => {
+            SectionInfoMsg::GetSectionResponse(GetSectionResponse::Redirect(sap)) => {
                 trace!("GetSectionResponse::Redirect, reboostrapping with provided peers");
                 // Disconnect from peer that sent us the redirect, connect to the new elders provided and
                 // request the section info again.
                 self.disconnect_from_peers(vec![src]).await?;
                 let endpoint = self.endpoint()?.clone();
                 let new_elders_addrs: Vec<SocketAddr> =
-                    elders.iter().map(|(_, addr)| *addr).collect();
+                    sap.elders.iter().map(|(_, addr)| *addr).collect();
                 self.qp2p
                     .update_bootstrap_contacts(new_elders_addrs.as_slice());
                 let boostrapped_peer = self
@@ -152,14 +150,13 @@ impl Session {
             SectionInfoMsg::SectionInfoUpdate(update) => {
                 let correlation_id = update.correlation_id;
                 error!("MessageId {:?} was interrupted due to infrastructure updates. This will most likely need to be sent again. Update was : {:?}", correlation_id, update);
-                if let SectionInfoError::TargetSectionInfoOutdated(info) = update.clone().error {
-                    trace!("Updated network info: ({:?})", info);
-                    self.update_session_info(&info).await?;
+                if let SectionInfoError::TargetSectionInfoOutdated(sap) = update.clone().error {
+                    trace!("Updated network info: ({:?})", sap);
+                    self.update_session_info(&sap).await?;
                 }
                 Ok(())
             }
             SectionInfoMsg::GetSectionResponse(GetSectionResponse::SectionInfoUpdate(_))
-            | SectionInfoMsg::GetSectionResponse(GetSectionResponse::NodeNotReachable)
             | SectionInfoMsg::GetSectionQuery { .. } => {
                 Err(Error::UnexpectedMessageOnJoin(format!(
                     "bootstrapping failed since an invalid response ({:?}) was received",
@@ -170,11 +167,11 @@ impl Session {
     }
 
     // Apply updated info to a network session, and trigger connections
-    async fn update_session_info(&mut self, info: &SectionInfo) -> Result<(), Error> {
-        let original_known_elders = self.all_known_elders.lock().await.clone();
+    async fn update_session_info(&mut self, sap: &SectionAuthorityProvider) -> Result<(), Error> {
+        let original_known_elders = self.all_known_elders.read().await.clone();
 
         // Change this once sn_messaging is updated
-        let received_elders = info
+        let received_elders = sap
             .elders
             .iter()
             .map(|(name, addr)| (*addr, *name))
@@ -188,23 +185,23 @@ impl Session {
 
         {
             // Update session key set
-            let mut keyset = self.section_key_set.lock().await;
-            if *keyset == Some(info.pk_set.clone()) {
+            let mut keyset = self.section_key_set.write().await;
+            if *keyset == Some(sap.public_key_set.clone()) {
                 trace!("We have previously received the key set already.");
                 return Ok(());
             }
-            *keyset = Some(info.pk_set.clone());
+            *keyset = Some(sap.public_key_set.clone());
         }
 
         {
             // update section prefix
-            let mut prefix = self.section_prefix.lock().await;
-            *prefix = Some(info.prefix);
+            let mut prefix = self.section_prefix.write().await;
+            *prefix = Some(sap.prefix);
         }
 
         {
             // Update session elders
-            let mut session_elders = self.all_known_elders.lock().await;
+            let mut session_elders = self.all_known_elders.write().await;
             *session_elders = received_elders.clone();
         }
 
@@ -232,67 +229,79 @@ impl Session {
 
     // Handle messages intended for client consumption (re: queries + commands)
     async fn handle_client_msg(&self, msg: ProcessMsg, src: SocketAddr) {
-        match msg {
-            ProcessMsg::QueryResponse {
-                response,
-                correlation_id,
-                ..
-            } => {
-                trace!(
-                    "Query response (correlation id: {}): {:?}",
+        debug!(
+            "===> ClientMsg with id {:?} received from {:?}",
+            msg.id(),
+            src
+        );
+        let queries = self.pending_queries.clone();
+        let transfers = self.pending_transfers.clone();
+        let error_sender = self.incoming_err_sender.clone();
+        let _ = tokio::spawn(async move {
+            debug!("Thread spawned to handle this client message");
+            match msg {
+                ProcessMsg::QueryResponse {
+                    response,
                     correlation_id,
-                    response
-                );
+                    ..
+                } => {
+                    debug!("Query response (relating to msgid: {})", correlation_id);
 
-                // Note that this doesn't remove the sender from here since multiple
-                // responses corresponding to the same message ID might arrive.
-                // Once we are satisfied with the response this is channel is discarded in
-                // ConnectionManager::send_query
-                if let Some(sender) = self.pending_queries.lock().await.get(&correlation_id) {
-                    trace!(
-                        "Sending response for query w/{} via channel.",
-                        correlation_id
-                    );
-                    let _ = sender.send(response).await;
-                } else {
-                    trace!("No channel found for {:?}", correlation_id);
-                }
-            }
-            ProcessMsg::Event {
-                event,
-                correlation_id,
-                ..
-            } => {
-                if let Event::TransferValidated { event, .. } = event {
-                    if let Some(sender) =
-                        self.pending_transfers.lock().await.get_mut(&correlation_id)
-                    {
-                        let _ = sender.send(Ok(event)).await;
+                    trace!("The received query response is {:?}", response);
+
+                    // Note that this doesn't remove the sender from here since multiple
+                    // responses corresponding to the same message ID might arrive.
+                    // Once we are satisfied with the response this is channel is discarded in
+                    // ConnectionManager::send_query
+                    if let Some(sender) = &queries.read().await.get(&correlation_id) {
+                        trace!(
+                            "Sending response for query w/{} via channel.",
+                            correlation_id
+                        );
+                        let _ = sender.send(response).await;
                     } else {
-                        warn!(
+                        trace!("No channel found for {:?}", correlation_id);
+                    }
+                }
+                ProcessMsg::Event {
+                    event,
+                    correlation_id,
+                    ..
+                } => {
+                    debug!("Event received to be processed: {:?}", correlation_id);
+                    trace!("Event received is: {:?}", event);
+
+                    if let Event::TransferValidated { event, .. } = event {
+                        let transfers = transfers.read().await;
+                        let sender = transfers.get(&correlation_id);
+                        if let Some(sender) = sender {
+                            let _ = sender.send(Ok(event)).await;
+                        } else {
+                            warn!(
                             "No transfer validation listener found for elder {:?} and message {:?}",
                             src, correlation_id
                         );
-                        warn!("It may be that this transfer is complete and the listener cleaned up already.");
-                        trace!("Event received was {:?}", event);
+                            warn!("It may be that this transfer is complete and the listener cleaned up already.");
+                            trace!("Event received was {:?}", event);
+                        }
                     }
                 }
-            }
-            ProcessMsg::CmdError {
-                error,
-                correlation_id,
-                ..
-            } => {
-                debug!(
-                    "Cmd Error was received for Message w/ID: {:?}, sending on error channel",
-                    correlation_id
-                );
-                trace!("Error received is: {:?}", error);
-                let _ = self.incoming_err_sender.send(error).await;
-            }
-            msg => {
-                warn!("Ignoring unexpected message type received: {:?}", msg);
-            }
-        };
+                ProcessMsg::CmdError {
+                    error,
+                    correlation_id,
+                    ..
+                } => {
+                    debug!(
+                        "Cmd Error was received for Message w/ID: {:?}, sending on error channel",
+                        correlation_id
+                    );
+                    trace!("Error received is: {:?}", error);
+                    let _ = error_sender.send(error).await;
+                }
+                msg => {
+                    warn!("Ignoring unexpected message type received: {:?}", msg);
+                }
+            };
+        });
     }
 }
